import { assert } from "chai";
import { Affiliation, CoreApiClient, WrappedWithMetaArrayPerson, } from "../api.js";
import { ActiveEntity, serializeFindQueryParams, } from "../model/queryParams.js";
import { client, getDefaultConfig, person1Uid, personInactiveUid, } from "./helpers.js";
describe("Querying", async () => {
    let allPersons;
    let clientWithInactiveEntities;
    before(async () => {
        allPersons = (await client.persons.find()).data;
        const defaultConfig = getDefaultConfig();
        defaultConfig.queryOptions = {
            includeInactive: [ActiveEntity.Affiliations, ActiveEntity.Consents],
            includePersonsWithoutChurchAffiliation: true,
        };
        clientWithInactiveEntities = new CoreApiClient(defaultConfig);
    });
    beforeEach(async () => {
        await client.refreshTestData();
    });
    it("serializeFindQueryParams", async () => {
        const params = serializeFindQueryParams({
            sort: [
                {
                    direction: "ascending",
                    path: "firstName",
                },
                "-lastName",
            ],
        });
        assert.equal(params.get("sort"), "firstName,-lastName");
    });
    it("sorting", async () => {
        const result = await client.persons.find({
            sort: [
                {
                    direction: "ascending",
                    path: "displayName",
                },
            ],
            limit: 2,
            offset: 1,
        });
        assert.equal(result.meta.total, allPersons.length);
        assert.equal(result.meta.skipped, 1);
        assert.equal(result.meta.limit, 2);
        assert.instanceOf(result, WrappedWithMetaArrayPerson);
        const sortedPersons = allPersons.sort((a, b) => a.displayName > b.displayName ? 1 : -1);
        assert.equal(result.data[0].uid, sortedPersons[1].uid);
        assert.equal(result.data[1].uid, sortedPersons[2].uid);
    });
    it("filtering", async () => {
        const result = await client.persons.find({
            filter: {
                displayName: {
                    _gte: "Philly Daly",
                },
            },
        });
        const expectedResult = allPersons.filter((p) => p.displayName >= "Philly Daly");
        assert.equal(result.meta.total, expectedResult.length);
    });
    it("filtering nested", async () => {
        const result = await client.persons.find({
            filter: {
                affiliations: {
                    person: {
                        displayName: {
                            _eq: "Philly Daly",
                        },
                    },
                },
            },
        });
        const expectedResult = allPersons.filter((p) => p.displayName >= "Philly Daly");
        assert.equal(result.meta.total, expectedResult.length);
    });
    it("search - find persons using a fuzzy search", async () => {
        const result = await client.persons.find({
            search: "Philli dali",
        });
        assert.isDefined(result);
        assert.lengthOf(result.data, 1);
        assert.equal(result.data[0].uid, person1Uid);
    });
    it("resolving", async () => {
        const result = await client.persons.find({
            fields: ["*", "affiliations.*"],
        });
        assert.instanceOf(result.data[0].affiliations?.[0], Affiliation);
    });
    it("options - include persons without church affiliation", async () => {
        const result = await client.persons.find({
            options: {
                includePersonsWithoutChurchAffiliation: true,
            },
        });
        const resultUids = result.data.map((r) => r.uid);
        assert.include(resultUids, personInactiveUid);
    });
    it("options - do not include persons without church affiliation", async () => {
        const result = await client.persons.find({
            options: {
                includePersonsWithoutChurchAffiliation: false,
            },
        });
        const resultUids = result.data.map((r) => r.uid);
        assert.notInclude(resultUids, personInactiveUid);
    });
    it("options - include inactive entities", async () => {
        const result = await client.affiliations.find({
            options: {
                includeInactive: [ActiveEntity.Affiliations],
            },
        });
        const inactiveAffiliation = result.data.find((a) => a.validFrom > new Date() || (a.validTo && a.validTo < new Date()));
        assert.isDefined(inactiveAffiliation);
    });
    it("options - do not include inactive entities", async () => {
        const result = await client.affiliations.find();
        const inactiveAffiliation = result.data.find((a) => a.validFrom > new Date() || (a.validTo && a.validTo < new Date()));
        assert.isUndefined(inactiveAffiliation);
    });
    it("options - include inactive by default with correct client", async () => {
        const result = await clientWithInactiveEntities.affiliations.find();
        const inactiveAffiliation = result.data.find((a) => a.validFrom > new Date() || (a.validTo && a.validTo < new Date()));
        assert.isDefined(inactiveAffiliation);
    });
    it("options - include persons without church affiliation with correct client", async () => {
        const result = await clientWithInactiveEntities.persons.get(personInactiveUid);
        assert.isDefined(result);
        assert.equal(result.data.uid, personInactiveUid);
    });
});
